#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, String, Polygon
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=167.6062992125984,height=40.66141732283465,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,5,18.37008)
		v0.add(Rect(0,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.909804,.878431,.466667,1),fillOpacity=None,strokeColor=Color(.909804,.878431,.466667,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(.689764,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.905805,.875592,.466976,1),fillOpacity=None,strokeColor=Color(.905805,.875592,.466976,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(1.379528,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.901806,.872752,.467285,1),fillOpacity=None,strokeColor=Color(.901806,.872752,.467285,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(2.069291,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.897807,.869912,.467594,1),fillOpacity=None,strokeColor=Color(.897807,.869912,.467594,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(2.759055,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.893809,.867072,.467903,1),fillOpacity=None,strokeColor=Color(.893809,.867072,.467903,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(3.448819,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.88981,.864233,.468212,1),fillOpacity=None,strokeColor=Color(.88981,.864233,.468212,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(4.138583,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.885811,.861393,.468521,1),fillOpacity=None,strokeColor=Color(.885811,.861393,.468521,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(4.828346,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.881812,.858553,.46883,1),fillOpacity=None,strokeColor=Color(.881812,.858553,.46883,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(5.51811,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.877813,.855713,.469139,1),fillOpacity=None,strokeColor=Color(.877813,.855713,.469139,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(6.207874,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.873814,.852874,.469448,1),fillOpacity=None,strokeColor=Color(.873814,.852874,.469448,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(6.897638,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.869816,.850034,.469758,1),fillOpacity=None,strokeColor=Color(.869816,.850034,.469758,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(7.587402,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.865817,.847194,.470067,1),fillOpacity=None,strokeColor=Color(.865817,.847194,.470067,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(8.277165,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.861818,.844354,.470376,1),fillOpacity=None,strokeColor=Color(.861818,.844354,.470376,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(8.966929,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.857819,.841515,.470685,1),fillOpacity=None,strokeColor=Color(.857819,.841515,.470685,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(9.656693,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.85382,.838675,.470994,1),fillOpacity=None,strokeColor=Color(.85382,.838675,.470994,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(10.34646,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.849821,.835835,.471303,1),fillOpacity=None,strokeColor=Color(.849821,.835835,.471303,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(11.03622,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.845822,.832995,.471612,1),fillOpacity=None,strokeColor=Color(.845822,.832995,.471612,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(11.72598,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.841824,.830156,.471921,1),fillOpacity=None,strokeColor=Color(.841824,.830156,.471921,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(12.41575,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.837825,.827316,.47223,1),fillOpacity=None,strokeColor=Color(.837825,.827316,.47223,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(13.10551,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.833826,.824476,.472539,1),fillOpacity=None,strokeColor=Color(.833826,.824476,.472539,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(13.79528,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.829827,.821636,.472848,1),fillOpacity=None,strokeColor=Color(.829827,.821636,.472848,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(14.48504,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.825828,.818796,.473158,1),fillOpacity=None,strokeColor=Color(.825828,.818796,.473158,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(15.1748,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.821829,.815957,.473467,1),fillOpacity=None,strokeColor=Color(.821829,.815957,.473467,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(15.86457,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.817831,.813117,.473776,1),fillOpacity=None,strokeColor=Color(.817831,.813117,.473776,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(16.55433,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.813832,.810277,.474085,1),fillOpacity=None,strokeColor=Color(.813832,.810277,.474085,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(17.24409,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.809833,.807437,.474394,1),fillOpacity=None,strokeColor=Color(.809833,.807437,.474394,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(17.93386,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.805834,.804598,.474703,1),fillOpacity=None,strokeColor=Color(.805834,.804598,.474703,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(18.62362,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.801835,.801758,.475012,1),fillOpacity=None,strokeColor=Color(.801835,.801758,.475012,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(19.31339,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.797836,.798918,.475321,1),fillOpacity=None,strokeColor=Color(.797836,.798918,.475321,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(20.00315,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.793838,.796078,.47563,1),fillOpacity=None,strokeColor=Color(.793838,.796078,.47563,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(0,0,20.69291,15.59055,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(10.34646,4.205276,'1',textAnchor='middle',fontName='Helvetica-Bold',fontSize=10,fillColor=Color(1,1,1,1),strokeColor=Color(0,0,0,1)))
		v0.add(Rect(22.8189,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.793838,.796078,.47563,1),fillOpacity=None,strokeColor=Color(.793838,.796078,.47563,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(23.50866,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.789839,.793239,.475939,1),fillOpacity=None,strokeColor=Color(.789839,.793239,.475939,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(24.19843,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.78584,.790399,.476248,1),fillOpacity=None,strokeColor=Color(.78584,.790399,.476248,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(24.88819,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.781841,.787559,.476558,1),fillOpacity=None,strokeColor=Color(.781841,.787559,.476558,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(25.57795,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.777842,.784719,.476867,1),fillOpacity=None,strokeColor=Color(.777842,.784719,.476867,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(26.26772,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.773843,.78188,.477176,1),fillOpacity=None,strokeColor=Color(.773843,.78188,.477176,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(26.95748,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.769844,.77904,.477485,1),fillOpacity=None,strokeColor=Color(.769844,.77904,.477485,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(27.64724,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.765846,.7762,.477794,1),fillOpacity=None,strokeColor=Color(.765846,.7762,.477794,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(28.33701,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.761847,.77336,.478103,1),fillOpacity=None,strokeColor=Color(.761847,.77336,.478103,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(29.02677,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.757848,.770521,.478412,1),fillOpacity=None,strokeColor=Color(.757848,.770521,.478412,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(29.71654,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.753849,.767681,.478721,1),fillOpacity=None,strokeColor=Color(.753849,.767681,.478721,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(30.4063,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.74985,.764841,.47903,1),fillOpacity=None,strokeColor=Color(.74985,.764841,.47903,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(31.09606,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.745851,.762001,.479339,1),fillOpacity=None,strokeColor=Color(.745851,.762001,.479339,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(31.78583,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.741853,.759162,.479648,1),fillOpacity=None,strokeColor=Color(.741853,.759162,.479648,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(32.47559,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.737854,.756322,.479958,1),fillOpacity=None,strokeColor=Color(.737854,.756322,.479958,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(33.16535,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.733855,.753482,.480267,1),fillOpacity=None,strokeColor=Color(.733855,.753482,.480267,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(33.85512,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.729856,.750642,.480576,1),fillOpacity=None,strokeColor=Color(.729856,.750642,.480576,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(34.54488,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.725857,.747803,.480885,1),fillOpacity=None,strokeColor=Color(.725857,.747803,.480885,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(35.23465,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.721858,.744963,.481194,1),fillOpacity=None,strokeColor=Color(.721858,.744963,.481194,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(35.92441,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.71786,.742123,.481503,1),fillOpacity=None,strokeColor=Color(.71786,.742123,.481503,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(36.61417,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.713861,.739283,.481812,1),fillOpacity=None,strokeColor=Color(.713861,.739283,.481812,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(37.30394,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.709862,.736444,.482121,1),fillOpacity=None,strokeColor=Color(.709862,.736444,.482121,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(37.9937,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.705863,.733604,.48243,1),fillOpacity=None,strokeColor=Color(.705863,.733604,.48243,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(38.68346,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.701864,.730764,.482739,1),fillOpacity=None,strokeColor=Color(.701864,.730764,.482739,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(39.37323,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.697865,.727924,.483048,1),fillOpacity=None,strokeColor=Color(.697865,.727924,.483048,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(40.06299,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.693867,.725085,.483357,1),fillOpacity=None,strokeColor=Color(.693867,.725085,.483357,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(40.75276,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.689868,.722245,.483667,1),fillOpacity=None,strokeColor=Color(.689868,.722245,.483667,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(41.44252,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.685869,.719405,.483976,1),fillOpacity=None,strokeColor=Color(.685869,.719405,.483976,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(42.13228,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.68187,.716565,.484285,1),fillOpacity=None,strokeColor=Color(.68187,.716565,.484285,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(42.82205,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.677871,.713725,.484594,1),fillOpacity=None,strokeColor=Color(.677871,.713725,.484594,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(22.8189,0,20.69291,15.59055,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(33.16535,4.205276,'2',textAnchor='middle',fontName='Helvetica-Bold',fontSize=10,fillColor=Color(1,1,1,1),strokeColor=Color(0,0,0,1)))
		v0.add(Rect(45.6378,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.677871,.713725,.484594,1),fillOpacity=None,strokeColor=Color(.677871,.713725,.484594,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(46.32756,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.673872,.710886,.484903,1),fillOpacity=None,strokeColor=Color(.673872,.710886,.484903,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(47.01732,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.669873,.708046,.485212,1),fillOpacity=None,strokeColor=Color(.669873,.708046,.485212,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(47.70709,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.665875,.705206,.485521,1),fillOpacity=None,strokeColor=Color(.665875,.705206,.485521,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(48.39685,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.661876,.702366,.48583,1),fillOpacity=None,strokeColor=Color(.661876,.702366,.48583,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(49.08661,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.657877,.699527,.486139,1),fillOpacity=None,strokeColor=Color(.657877,.699527,.486139,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(49.77638,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.653878,.696687,.486448,1),fillOpacity=None,strokeColor=Color(.653878,.696687,.486448,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(50.46614,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.649879,.693847,.486757,1),fillOpacity=None,strokeColor=Color(.649879,.693847,.486757,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(51.15591,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.64588,.691007,.487067,1),fillOpacity=None,strokeColor=Color(.64588,.691007,.487067,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(51.84567,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.641882,.688168,.487376,1),fillOpacity=None,strokeColor=Color(.641882,.688168,.487376,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(52.53543,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.637883,.685328,.487685,1),fillOpacity=None,strokeColor=Color(.637883,.685328,.487685,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(53.2252,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.633884,.682488,.487994,1),fillOpacity=None,strokeColor=Color(.633884,.682488,.487994,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(53.91496,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.629885,.679648,.488303,1),fillOpacity=None,strokeColor=Color(.629885,.679648,.488303,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(54.60472,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.625886,.676809,.488612,1),fillOpacity=None,strokeColor=Color(.625886,.676809,.488612,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(55.29449,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.621887,.673969,.488921,1),fillOpacity=None,strokeColor=Color(.621887,.673969,.488921,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(55.98425,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.617889,.671129,.48923,1),fillOpacity=None,strokeColor=Color(.617889,.671129,.48923,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(56.67402,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.61389,.668289,.489539,1),fillOpacity=None,strokeColor=Color(.61389,.668289,.489539,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(57.36378,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.609891,.66545,.489848,1),fillOpacity=None,strokeColor=Color(.609891,.66545,.489848,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(58.05354,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.605892,.66261,.490157,1),fillOpacity=None,strokeColor=Color(.605892,.66261,.490157,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(58.74331,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.601893,.65977,.490467,1),fillOpacity=None,strokeColor=Color(.601893,.65977,.490467,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(59.43307,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.597894,.65693,.490776,1),fillOpacity=None,strokeColor=Color(.597894,.65693,.490776,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(60.12283,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.593895,.654091,.491085,1),fillOpacity=None,strokeColor=Color(.593895,.654091,.491085,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(60.8126,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.589897,.651251,.491394,1),fillOpacity=None,strokeColor=Color(.589897,.651251,.491394,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(61.50236,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.585898,.648411,.491703,1),fillOpacity=None,strokeColor=Color(.585898,.648411,.491703,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(62.19213,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.581899,.645571,.492012,1),fillOpacity=None,strokeColor=Color(.581899,.645571,.492012,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(62.88189,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.5779,.642732,.492321,1),fillOpacity=None,strokeColor=Color(.5779,.642732,.492321,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(63.57165,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.573901,.639892,.49263,1),fillOpacity=None,strokeColor=Color(.573901,.639892,.49263,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(64.26142,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.569902,.637052,.492939,1),fillOpacity=None,strokeColor=Color(.569902,.637052,.492939,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(64.95118,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.565904,.634212,.493248,1),fillOpacity=None,strokeColor=Color(.565904,.634212,.493248,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(65.64094,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.561905,.631373,.493557,1),fillOpacity=None,strokeColor=Color(.561905,.631373,.493557,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(45.6378,0,20.69291,15.59055,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(55.98425,4.205276,'3',textAnchor='middle',fontName='Helvetica-Bold',fontSize=10,fillColor=Color(1,1,1,1),strokeColor=Color(0,0,0,1)))
		v0.add(Rect(68.45669,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.561905,.631373,.493557,1),fillOpacity=None,strokeColor=Color(.561905,.631373,.493557,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(69.14646,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.557906,.628533,.493867,1),fillOpacity=None,strokeColor=Color(.557906,.628533,.493867,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(69.83622,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.553907,.625693,.494176,1),fillOpacity=None,strokeColor=Color(.553907,.625693,.494176,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(70.52598,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.549908,.622853,.494485,1),fillOpacity=None,strokeColor=Color(.549908,.622853,.494485,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(71.21575,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.545909,.620014,.494794,1),fillOpacity=None,strokeColor=Color(.545909,.620014,.494794,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(71.90551,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.541911,.617174,.495103,1),fillOpacity=None,strokeColor=Color(.541911,.617174,.495103,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(72.59528,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.537912,.614334,.495412,1),fillOpacity=None,strokeColor=Color(.537912,.614334,.495412,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(73.28504,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.533913,.611494,.495721,1),fillOpacity=None,strokeColor=Color(.533913,.611494,.495721,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(73.9748,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.529914,.608654,.49603,1),fillOpacity=None,strokeColor=Color(.529914,.608654,.49603,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(74.66457,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.525915,.605815,.496339,1),fillOpacity=None,strokeColor=Color(.525915,.605815,.496339,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(75.35433,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.521916,.602975,.496648,1),fillOpacity=None,strokeColor=Color(.521916,.602975,.496648,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(76.04409,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.517918,.600135,.496957,1),fillOpacity=None,strokeColor=Color(.517918,.600135,.496957,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(76.73386,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.513919,.597295,.497266,1),fillOpacity=None,strokeColor=Color(.513919,.597295,.497266,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(77.42362,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.50992,.594456,.497576,1),fillOpacity=None,strokeColor=Color(.50992,.594456,.497576,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(78.11339,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.505921,.591616,.497885,1),fillOpacity=None,strokeColor=Color(.505921,.591616,.497885,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(78.80315,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.501922,.588776,.498194,1),fillOpacity=None,strokeColor=Color(.501922,.588776,.498194,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(79.49291,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.497923,.585936,.498503,1),fillOpacity=None,strokeColor=Color(.497923,.585936,.498503,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(80.18268,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.493924,.583097,.498812,1),fillOpacity=None,strokeColor=Color(.493924,.583097,.498812,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(80.87244,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.489926,.580257,.499121,1),fillOpacity=None,strokeColor=Color(.489926,.580257,.499121,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(81.5622,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.485927,.577417,.49943,1),fillOpacity=None,strokeColor=Color(.485927,.577417,.49943,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(82.25197,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.481928,.574577,.499739,1),fillOpacity=None,strokeColor=Color(.481928,.574577,.499739,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(82.94173,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.477929,.571738,.500048,1),fillOpacity=None,strokeColor=Color(.477929,.571738,.500048,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(83.6315,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.47393,.568898,.500357,1),fillOpacity=None,strokeColor=Color(.47393,.568898,.500357,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(84.32126,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.469931,.566058,.500666,1),fillOpacity=None,strokeColor=Color(.469931,.566058,.500666,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(85.01102,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.465933,.563218,.500976,1),fillOpacity=None,strokeColor=Color(.465933,.563218,.500976,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(85.70079,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.461934,.560379,.501285,1),fillOpacity=None,strokeColor=Color(.461934,.560379,.501285,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(86.39055,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.457935,.557539,.501594,1),fillOpacity=None,strokeColor=Color(.457935,.557539,.501594,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(87.08031,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.453936,.554699,.501903,1),fillOpacity=None,strokeColor=Color(.453936,.554699,.501903,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(87.77008,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.449937,.551859,.502212,1),fillOpacity=None,strokeColor=Color(.449937,.551859,.502212,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(88.45984,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.445938,.54902,.502521,1),fillOpacity=None,strokeColor=Color(.445938,.54902,.502521,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(68.45669,0,20.69291,15.59055,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(78.80315,4.205276,'4',textAnchor='middle',fontName='Helvetica-Bold',fontSize=10,fillColor=Color(1,1,1,1),strokeColor=Color(0,0,0,1)))
		v0.add(Rect(91.27559,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.445938,.54902,.502521,1),fillOpacity=None,strokeColor=Color(.445938,.54902,.502521,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(91.96535,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.44194,.54618,.50283,1),fillOpacity=None,strokeColor=Color(.44194,.54618,.50283,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(92.65512,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.437941,.54334,.503139,1),fillOpacity=None,strokeColor=Color(.437941,.54334,.503139,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(93.34488,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.433942,.5405,.503448,1),fillOpacity=None,strokeColor=Color(.433942,.5405,.503448,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(94.03465,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.429943,.537661,.503757,1),fillOpacity=None,strokeColor=Color(.429943,.537661,.503757,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(94.72441,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.425944,.534821,.504066,1),fillOpacity=None,strokeColor=Color(.425944,.534821,.504066,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(95.41417,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.421945,.531981,.504376,1),fillOpacity=None,strokeColor=Color(.421945,.531981,.504376,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(96.10394,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.417946,.529141,.504685,1),fillOpacity=None,strokeColor=Color(.417946,.529141,.504685,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(96.7937,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.413948,.526302,.504994,1),fillOpacity=None,strokeColor=Color(.413948,.526302,.504994,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(97.48346,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.409949,.523462,.505303,1),fillOpacity=None,strokeColor=Color(.409949,.523462,.505303,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(98.17323,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.40595,.520622,.505612,1),fillOpacity=None,strokeColor=Color(.40595,.520622,.505612,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(98.86299,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.401951,.517782,.505921,1),fillOpacity=None,strokeColor=Color(.401951,.517782,.505921,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(99.55276,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.397952,.514943,.50623,1),fillOpacity=None,strokeColor=Color(.397952,.514943,.50623,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(100.2425,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.393953,.512103,.506539,1),fillOpacity=None,strokeColor=Color(.393953,.512103,.506539,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(100.9323,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.389955,.509263,.506848,1),fillOpacity=None,strokeColor=Color(.389955,.509263,.506848,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(101.622,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.385956,.506423,.507157,1),fillOpacity=None,strokeColor=Color(.385956,.506423,.507157,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(102.3118,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.381957,.503584,.507466,1),fillOpacity=None,strokeColor=Color(.381957,.503584,.507466,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(103.0016,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.377958,.500744,.507776,1),fillOpacity=None,strokeColor=Color(.377958,.500744,.507776,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(103.6913,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.373959,.497904,.508085,1),fillOpacity=None,strokeColor=Color(.373959,.497904,.508085,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(104.3811,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.36996,.495064,.508394,1),fillOpacity=None,strokeColor=Color(.36996,.495064,.508394,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(105.0709,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.365962,.492224,.508703,1),fillOpacity=None,strokeColor=Color(.365962,.492224,.508703,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(105.7606,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.361963,.489385,.509012,1),fillOpacity=None,strokeColor=Color(.361963,.489385,.509012,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(106.4504,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.357964,.486545,.509321,1),fillOpacity=None,strokeColor=Color(.357964,.486545,.509321,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(107.1402,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.353965,.483705,.50963,1),fillOpacity=None,strokeColor=Color(.353965,.483705,.50963,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(107.8299,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.349966,.480865,.509939,1),fillOpacity=None,strokeColor=Color(.349966,.480865,.509939,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(108.5197,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.345967,.478026,.510248,1),fillOpacity=None,strokeColor=Color(.345967,.478026,.510248,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(109.2094,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.341969,.475186,.510557,1),fillOpacity=None,strokeColor=Color(.341969,.475186,.510557,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(109.8992,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.33797,.472346,.510866,1),fillOpacity=None,strokeColor=Color(.33797,.472346,.510866,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(110.589,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.333971,.469506,.511176,1),fillOpacity=None,strokeColor=Color(.333971,.469506,.511176,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(111.2787,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.329972,.466667,.511485,1),fillOpacity=None,strokeColor=Color(.329972,.466667,.511485,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(91.27559,0,20.69291,15.59055,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(101.622,4.205276,'5',textAnchor='middle',fontName='Helvetica-Bold',fontSize=10,fillColor=Color(1,1,1,1),strokeColor=Color(0,0,0,1)))
		v0.add(Rect(114.0945,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.329972,.466667,.511485,1),fillOpacity=None,strokeColor=Color(.329972,.466667,.511485,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(114.7843,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.325973,.463827,.511794,1),fillOpacity=None,strokeColor=Color(.325973,.463827,.511794,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(115.474,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.321974,.460987,.512103,1),fillOpacity=None,strokeColor=Color(.321974,.460987,.512103,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(116.1638,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.317975,.458147,.512412,1),fillOpacity=None,strokeColor=Color(.317975,.458147,.512412,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(116.8535,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.313977,.455308,.512721,1),fillOpacity=None,strokeColor=Color(.313977,.455308,.512721,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(117.5433,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.309978,.452468,.51303,1),fillOpacity=None,strokeColor=Color(.309978,.452468,.51303,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(118.2331,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.305979,.449628,.513339,1),fillOpacity=None,strokeColor=Color(.305979,.449628,.513339,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(118.9228,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.30198,.446788,.513648,1),fillOpacity=None,strokeColor=Color(.30198,.446788,.513648,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(119.6126,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.297981,.443949,.513957,1),fillOpacity=None,strokeColor=Color(.297981,.443949,.513957,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(120.3024,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.293982,.441109,.514266,1),fillOpacity=None,strokeColor=Color(.293982,.441109,.514266,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(120.9921,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.289984,.438269,.514575,1),fillOpacity=None,strokeColor=Color(.289984,.438269,.514575,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(121.6819,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.285985,.435429,.514885,1),fillOpacity=None,strokeColor=Color(.285985,.435429,.514885,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(122.3717,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.281986,.43259,.515194,1),fillOpacity=None,strokeColor=Color(.281986,.43259,.515194,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(123.0614,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.277987,.42975,.515503,1),fillOpacity=None,strokeColor=Color(.277987,.42975,.515503,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(123.7512,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.273988,.42691,.515812,1),fillOpacity=None,strokeColor=Color(.273988,.42691,.515812,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(124.4409,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.269989,.42407,.516121,1),fillOpacity=None,strokeColor=Color(.269989,.42407,.516121,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(125.1307,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.265991,.421231,.51643,1),fillOpacity=None,strokeColor=Color(.265991,.421231,.51643,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(125.8205,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.261992,.418391,.516739,1),fillOpacity=None,strokeColor=Color(.261992,.418391,.516739,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(126.5102,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.257993,.415551,.517048,1),fillOpacity=None,strokeColor=Color(.257993,.415551,.517048,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(127.2,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.253994,.412711,.517357,1),fillOpacity=None,strokeColor=Color(.253994,.412711,.517357,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(127.8898,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.249995,.409872,.517666,1),fillOpacity=None,strokeColor=Color(.249995,.409872,.517666,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(128.5795,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.245996,.407032,.517975,1),fillOpacity=None,strokeColor=Color(.245996,.407032,.517975,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(129.2693,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.241997,.404192,.518285,1),fillOpacity=None,strokeColor=Color(.241997,.404192,.518285,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(129.9591,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.237999,.401352,.518594,1),fillOpacity=None,strokeColor=Color(.237999,.401352,.518594,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(130.6488,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.234,.398513,.518903,1),fillOpacity=None,strokeColor=Color(.234,.398513,.518903,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(131.3386,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.230001,.395673,.519212,1),fillOpacity=None,strokeColor=Color(.230001,.395673,.519212,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(132.0283,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.226002,.392833,.519521,1),fillOpacity=None,strokeColor=Color(.226002,.392833,.519521,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(132.7181,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.222003,.389993,.51983,1),fillOpacity=None,strokeColor=Color(.222003,.389993,.51983,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(133.4079,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.218004,.387153,.520139,1),fillOpacity=None,strokeColor=Color(.218004,.387153,.520139,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(134.0976,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.214006,.384314,.520448,1),fillOpacity=None,strokeColor=Color(.214006,.384314,.520448,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(114.0945,0,20.69291,15.59055,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(124.4409,4.205276,'6',textAnchor='middle',fontName='Helvetica-Bold',fontSize=10,fillColor=Color(1,1,1,1),strokeColor=Color(0,0,0,1)))
		v0.add(Rect(136.9134,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.214006,.384314,.520448,1),fillOpacity=None,strokeColor=Color(.214006,.384314,.520448,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(137.6031,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.210007,.381474,.520757,1),fillOpacity=None,strokeColor=Color(.210007,.381474,.520757,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(138.2929,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.206008,.378634,.521066,1),fillOpacity=None,strokeColor=Color(.206008,.378634,.521066,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(138.9827,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.202009,.375794,.521375,1),fillOpacity=None,strokeColor=Color(.202009,.375794,.521375,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(139.6724,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.19801,.372955,.521685,1),fillOpacity=None,strokeColor=Color(.19801,.372955,.521685,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(140.3622,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.194011,.370115,.521994,1),fillOpacity=None,strokeColor=Color(.194011,.370115,.521994,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(141.052,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.190013,.367275,.522303,1),fillOpacity=None,strokeColor=Color(.190013,.367275,.522303,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(141.7417,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.186014,.364435,.522612,1),fillOpacity=None,strokeColor=Color(.186014,.364435,.522612,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(142.4315,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.182015,.361596,.522921,1),fillOpacity=None,strokeColor=Color(.182015,.361596,.522921,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(143.1213,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.178016,.358756,.52323,1),fillOpacity=None,strokeColor=Color(.178016,.358756,.52323,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(143.811,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.174017,.355916,.523539,1),fillOpacity=None,strokeColor=Color(.174017,.355916,.523539,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(144.5008,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.170018,.353076,.523848,1),fillOpacity=None,strokeColor=Color(.170018,.353076,.523848,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(145.1906,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.16602,.350237,.524157,1),fillOpacity=None,strokeColor=Color(.16602,.350237,.524157,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(145.8803,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.162021,.347397,.524466,1),fillOpacity=None,strokeColor=Color(.162021,.347397,.524466,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(146.5701,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.158022,.344557,.524775,1),fillOpacity=None,strokeColor=Color(.158022,.344557,.524775,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(147.2598,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.154023,.341717,.525085,1),fillOpacity=None,strokeColor=Color(.154023,.341717,.525085,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(147.9496,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.150024,.338878,.525394,1),fillOpacity=None,strokeColor=Color(.150024,.338878,.525394,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(148.6394,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.146025,.336038,.525703,1),fillOpacity=None,strokeColor=Color(.146025,.336038,.525703,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(149.3291,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.142026,.333198,.526012,1),fillOpacity=None,strokeColor=Color(.142026,.333198,.526012,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(150.0189,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.138028,.330358,.526321,1),fillOpacity=None,strokeColor=Color(.138028,.330358,.526321,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(150.7087,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.134029,.327519,.52663,1),fillOpacity=None,strokeColor=Color(.134029,.327519,.52663,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(151.3984,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.13003,.324679,.526939,1),fillOpacity=None,strokeColor=Color(.13003,.324679,.526939,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(152.0882,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.126031,.321839,.527248,1),fillOpacity=None,strokeColor=Color(.126031,.321839,.527248,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(152.778,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.122032,.318999,.527557,1),fillOpacity=None,strokeColor=Color(.122032,.318999,.527557,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(153.4677,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.118033,.31616,.527866,1),fillOpacity=None,strokeColor=Color(.118033,.31616,.527866,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(154.1575,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.114035,.31332,.528175,1),fillOpacity=None,strokeColor=Color(.114035,.31332,.528175,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(154.8472,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.110036,.31048,.528484,1),fillOpacity=None,strokeColor=Color(.110036,.31048,.528484,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(155.537,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.106037,.30764,.528794,1),fillOpacity=None,strokeColor=Color(.106037,.30764,.528794,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(156.2268,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.102038,.304801,.529103,1),fillOpacity=None,strokeColor=Color(.102038,.304801,.529103,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(156.9165,0,.689764,15.59055,rx=0,ry=0,fillColor=Color(.098039,.301961,.529412,1),fillOpacity=None,strokeColor=Color(.098039,.301961,.529412,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Rect(136.9134,0,20.69291,15.59055,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(147.2598,4.205276,'7',textAnchor='middle',fontName='Helvetica-Bold',fontSize=10,fillColor=Color(1,1,1,1),strokeColor=Color(0,0,0,1)))
		v0.add(Polygon(points=[147.2598,13.88976,141.874,17.29134,152.6457,17.29134,147.2598,13.88976],fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(Polygon(points=[147.2598,1.700787,141.874,-1.700787,152.6457,-1.700787,147.2598,1.700787],fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.58,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0.add(String(157.6063,-13.37008,'Source: ReportLab',textAnchor='end',fontName='Helvetica-Oblique',fontSize=6,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



"""Test TrueType font subsetting & embedding code.

This test uses a sample font (Vera.ttf) taken from Bitstream which is called Vera
Serif Regular and is covered under the license in ../fonts/bitstream-vera-license.txt.
"""
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation, NearTestCase, rlSkipUnless
if __name__=='__main__':
    setOutDir(__name__)
import unittest, os
from io import BytesIO
from reportlab.pdfgen.canvas import Canvas
from reportlab.pdfgen.textobject import PDFTextObject
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.pdfdoc import PDFDocument, PDFError
from reportlab.pdfbase.ttfonts import TTFont, TTFontFace, TTFontFile, TTFOpenFile, \
                                      TTFontParser, TTFontMaker, TTFError, \
                                      makeToUnicodeCMap, \
                                      FF_SYMBOLIC, FF_NONSYMBOLIC, \
                                      calcChecksum, add32, uharfbuzz, shapeFragWord, \
                                      ShapedFragWord, ShapedStr, ShapeData, _sdSimple, \
                                      freshTTFont
from reportlab.platypus.paragraph import Paragraph, _HSFrag
from reportlab.lib.styles import getSampleStyleSheet
import zlib, base64
from reportlab import rl_config
from reportlab.lib.utils import int2Byte
from reportlab.lib.abag import ABag

try:
    TTFont("DejaVuSans", "DejaVuSans.ttf")
except:
    haveDejaVuSans = False
else:
    haveDejaVuSans = True

def utf8(code):
    "Convert a given UCS character index into UTF-8"
    return chr(code).encode('utf8')

def _simple_subset_generation(fn,npages,alter=0,fonts=('Vera','VeraBI')):
    c = Canvas(outputfile(fn))
    c.setFont('Helvetica', 30)
    c.drawString(100,700, 'Unicode TrueType Font Test %d pages' % npages)
    # Draw a table of Unicode characters
    for p in range(npages):
        for fontName in fonts:
            c.setFont(fontName, 10)
            for i in range(32):
                for j in range(32):
                    ch = chr(i * 32 + j+p*alter)
                    c.drawString(80 + j * 13 + int(j / 16.0) * 4, 600 - i * 13 - int(i / 8.0) * 8, ch)
        c.showPage()
    c.save()

def show_all_glyphs(fn,fontName='Vera'):
    c = Canvas(outputfile(fn))
    c.setFont('Helvetica', 20)
    c.drawString(72,c._pagesize[1]-30, 'Unicode TrueType Font Test %s' % fontName)
    from reportlab.pdfbase.pdfmetrics import _fonts
    font = _fonts[fontName]
    doc = c._doc
    kfunc = font.face.charToGlyph.keys
    for s in sorted(list(kfunc())):
        if s<0x10000:
            font.splitString(chr(s),doc)
    state = font.state[doc]
    cn = {}
    #print('len(assignments)=%d'%  len(state.assignments))
    nzero = 0
    ifunc = state.assignments.items
    for code, n in sorted(list(ifunc())):
        if code==0: nzero += 1
        cn[n] = chr(code)
    if nzero>1: print('%s there were %d zero codes' % (fontName,nzero))


    ymin = 10*12
    y = y0 = c._pagesize[1] - 72
    for nss,subset in enumerate(state.subsets):
        if y<ymin:
            c.showPage()
            y = y0
        c.setFont('Helvetica', 10)
        x = 72
        c.drawString(x,y,'Subset %d len=%d' % (nss,len(subset)))
        #print('Subset %d len=%d' % (nss,len(subset)))
        c.setFont(fontName, 10)
        for i, code in enumerate(subset):
            if i%32 == 0:
                y -= 12
                x = 72
            c.drawString(x,y,chr(code))
            x += 13
        y -= 18

    c.showPage()
    c.save()

class TTFontsTestCase(unittest.TestCase):
    "Make documents with TrueType fonts"

    def testTTF(self):
        "Test PDF generation with TrueType fonts"
        pdfmetrics.registerFont(TTFont("Vera", "Vera.ttf"))
        pdfmetrics.registerFont(TTFont("VeraBI", "VeraBI.ttf"))
        if haveDejaVuSans:
            pdfmetrics.registerFont(freshTTFont('DejaVuSans', 'DejaVuSans.ttf'))
            show_all_glyphs('test_pdfbase_ttfonts_dejavusans.pdf',fontName='DejaVuSans')
        show_all_glyphs('test_pdfbase_ttfonts_vera.pdf',fontName='Vera')
        _simple_subset_generation('test_pdfbase_ttfonts1.pdf',1)
        _simple_subset_generation('test_pdfbase_ttfonts3.pdf',3)
        _simple_subset_generation('test_pdfbase_ttfonts35.pdf',3,5)

        # Do it twice with the same font object
        c = Canvas(outputfile('test_pdfbase_ttfontsadditional.pdf'))
        # Draw a table of Unicode characters
        c.setFont('Vera', 10)
        c.drawString(100, 700, b'Hello, ' + utf8(0xffee))
        c.save()

    def testSameTTFDifferentName(self):
        "Test PDF generation with TrueType fonts"
        pdfmetrics.registerFont(TTFont("Vera", "Vera.ttf"))
        pdfmetrics.registerFont(TTFont("MyVera", "Vera.ttf"))

        # Do it twice with the same font object
        c = Canvas(outputfile('test_pdfbase_ttfontsduplicate.pdf'))
        # Draw a table of Unicode characters
        c.setFont('Vera', 10)
        c.drawString(100, 700, b'Hello World')
        c.setFont('MyVera', 10)
        c.drawString(100, 688, b'Hello World')
        c.save()

class TTFontFileTestCase(NearTestCase):
    "Tests TTFontFile, TTFontParser and TTFontMaker classes"

    def testFontFileFailures(self):
        "Tests TTFontFile constructor error checks"
        self.assertRaises(TTFError, TTFontFile, "nonexistent file")
        self.assertRaises(TTFError, TTFontFile, BytesIO(b""))
        self.assertRaises(TTFError, TTFontFile, BytesIO(b"invalid signature"))
        self.assertRaises(TTFError, TTFontFile, BytesIO(b"OTTO - OpenType not supported yet"))
        self.assertRaises(TTFError, TTFontFile, BytesIO(b"\0\1\0\0"))

    def testFontFileReads(self):
        "Tests TTFontParset.read_xxx"

        class FakeTTFontFile(TTFontParser):
            def __init__(self, data):
                self._ttf_data = data
                self._pos = 0

        ttf = FakeTTFontFile(b"\x81\x02\x03\x04" b"\x85\x06" b"ABCD" b"\x7F\xFF" b"\x80\x00" b"\xFF\xFF")
        self.assertEqual(ttf.read_ulong(), 0x81020304) # big-endian
        self.assertEqual(ttf._pos, 4)
        self.assertEqual(ttf.read_ushort(), 0x8506)
        self.assertEqual(ttf._pos, 6)
        self.assertEqual(ttf.read_tag(), 'ABCD')
        self.assertEqual(ttf._pos, 10)
        self.assertEqual(ttf.read_short(), 0x7FFF)
        self.assertEqual(ttf.read_short(), -0x8000)
        self.assertEqual(ttf.read_short(), -1)

    def testFontFile(self):
        "Tests TTFontFile and TTF parsing code"
        ttf = TTFontFile("Vera.ttf")
        self.assertEqual(ttf.name, b"BitstreamVeraSans-Roman")
        self.assertEqual(ttf.flags, FF_SYMBOLIC)
        self.assertEqual(ttf.italicAngle, 0.0)
        self.assertNear(ttf.ascent,759.765625)
        self.assertNear(ttf.descent,-240.234375)
        self.assertEqual(ttf.capHeight, 759.765625)
        self.assertNear(ttf.bbox, [-183.10546875, -235.83984375, 1287.109375, 928.22265625])
        self.assertEqual(ttf.stemV, 87)
        self.assertEqual(ttf.defaultWidth, 600.09765625)

    def testAdd32(self):
        "Test add32"
        self.assertEqual(add32(10, -6), 4)
        self.assertEqual(add32(6, -10), -4&0xFFFFFFFF)
        self.assertEqual(add32(0x80000000, -1), 0x7FFFFFFF)
        self.assertEqual(add32(0x7FFFFFFF, 1), 0x80000000)

    def testChecksum(self):
        "Test calcChecksum function"
        self.assertEqual(calcChecksum(b""), 0)
        self.assertEqual(calcChecksum(b"\1"), 0x01000000)
        self.assertEqual(calcChecksum(b"\x01\x02\x03\x04\x10\x20\x30\x40"), 0x11223344)
        self.assertEqual(calcChecksum(b"\x81"), 0x81000000)
        self.assertEqual(calcChecksum(b"\x81\x02"), 0x81020000)
        self.assertEqual(calcChecksum(b"\x81\x02\x03"), 0x81020300)
        self.assertEqual(calcChecksum(b"\x81\x02\x03\x04"), 0x81020304)
        self.assertEqual(calcChecksum(b"\x81\x02\x03\x04\x05"), 0x86020304)
        self.assertEqual(calcChecksum(b"\x41\x02\x03\x04\xD0\x20\x30\x40"), 0x11223344)
        self.assertEqual(calcChecksum(b"\xD1\x02\x03\x04\x40\x20\x30\x40"), 0x11223344)
        self.assertEqual(calcChecksum(b"\x81\x02\x03\x04\x90\x20\x30\x40"), 0x11223344)
        self.assertEqual(calcChecksum(b"\x7F\xFF\xFF\xFF\x00\x00\x00\x01"), 0x80000000)

    def testFontFileChecksum(self):
        "Tests TTFontFile and TTF parsing code"
        F = TTFOpenFile("Vera.ttf")[1].read()
        TTFontFile(BytesIO(F), validate=1) # should not fail
        F1 = F[:12345] + b"\xFF" + F[12346:] # change one byte
        self.assertRaises(TTFError, TTFontFile, BytesIO(F1), validate=1)
        F1 = F[:8] + b"\xFF" + F[9:] # change one byte
        self.assertRaises(TTFError, TTFontFile, BytesIO(F1), validate=1)

    def testSubsetting(self):
        "Tests TTFontFile and TTF parsing code"
        ttf = TTFontFile("Vera.ttf")
        subset = ttf.makeSubset([0x41, 0x42])
        subset = TTFontFile(BytesIO(subset), 0)
        for tag in ('cmap', 'head', 'hhea', 'hmtx', 'maxp', 'name', 'OS/2',
                    'post', 'cvt ', 'fpgm', 'glyf', 'loca', 'prep'):
            self.assertTrue(subset.get_table(tag))

        subset.seek_table('loca')
        for n in range(4):
            pos = subset.read_ushort()    # this is actually offset / 2
            self.assertFalse(pos % 2 != 0, "glyph %d at +%d should be long aligned" % (n, pos * 2))

        self.assertEqual(subset.name, b"BitstreamVeraSans-Roman")
        self.assertEqual(subset.flags, FF_SYMBOLIC)
        self.assertEqual(subset.italicAngle, 0.0)
        self.assertNear(subset.ascent,759.765625)
        self.assertNear(subset.descent,-240.234375)
        self.assertEqual(subset.capHeight, 759.765625)
        self.assertNear(subset.bbox, [-183.10546875, -235.83984375, 1287.109375, 928.22265625])
        self.assertEqual(subset.stemV, 87)

    def testFontMaker(self):
        "Tests TTFontMaker class"
        ttf = TTFontMaker()
        ttf.add("ABCD", b"xyzzy")
        ttf.add("QUUX", b"123")
        ttf.add("head", b"12345678xxxx")
        stm = ttf.makeStream()
        ttf = TTFontParser(BytesIO(stm), 0)
        self.assertEqual(ttf.get_table("ABCD"), b"xyzzy")
        self.assertEqual(ttf.get_table("QUUX"), b"123")


class TTFontFaceTestCase(unittest.TestCase):
    "Tests TTFontFace class"

    def testAddSubsetObjects(self):
        "Tests TTFontFace.addSubsetObjects"
        face = TTFontFace("Vera.ttf")
        doc = PDFDocument()
        fontDescriptor = face.addSubsetObjects(doc, "TestFont", [ 0x78, 0x2017 ])
        fontDescriptor = doc.idToObject[fontDescriptor.name].dict
        self.assertEqual(fontDescriptor['Type'], '/FontDescriptor')
        self.assertEqual(fontDescriptor['Ascent'], face.ascent)
        self.assertEqual(fontDescriptor['CapHeight'], face.capHeight)
        self.assertEqual(fontDescriptor['Descent'], face.descent)
        self.assertEqual(fontDescriptor['Flags'], (face.flags & ~FF_NONSYMBOLIC) | FF_SYMBOLIC)
        self.assertEqual(fontDescriptor['FontName'], "/TestFont")
        self.assertEqual(fontDescriptor['FontBBox'].sequence, face.bbox)
        self.assertEqual(fontDescriptor['ItalicAngle'], face.italicAngle)
        self.assertEqual(fontDescriptor['StemV'], face.stemV)
        fontFile = fontDescriptor['FontFile2']
        fontFile = doc.idToObject[fontFile.name]
        self.assertTrue(fontFile.content != "")

class TTFontTestCase(NearTestCase):
    "Tests TTFont class"

    def testStringWidth(self):
        "Test TTFont.stringWidth"
        font = TTFont("Vera", "Vera.ttf")
        self.assertTrue(font.stringWidth("test", 10) > 0)
        width = font.stringWidth(utf8(0x2260) * 2, 1000)
        expected = font.face.getCharWidth(0x2260) * 2
        self.assertNear(width,expected)

    def testTTFontFromBytesIO(self):
        "Test loading TTFont from in-memory file"
        # Direct source: https://github.com/tmbdev/hocr-tools/blob/master/hocr-pdf
        # Glyphless variation of vedaal's invisible font retrieved from
        # http://www.angelfire.com/pr/pgpf/if.html, which says:
        # 'Invisible font' is unrestricted freeware. Enjoy, Improve, Distribute freely
        font = """
        eJzdlk1sG0UUx/+zs3btNEmrUKpCPxikSqRS4jpfFURUagmkEQQoiRXgAl07Y3vL2mvt2ml8APXG
        hQPiUEGEVDhWVHyIC1REPSAhBOWA+BCgSoULUqsKcWhVBKjhzfPU+VCi3Flrdn7vzZv33ryZ3TUE
        gC6chsTx8fHck1ONd98D0jnS7jn26GPjyMIleZhk9fT0wcHFl1/9GRDPkTxTqHg1dMkzJH9CbbTk
        xbWlJfKEdB+Np0pBswi+nH/Nvay92VtfJp4nvEztUJkUHXsdksUOkveXK/X5FNuLD838ICx4dv4N
        I1e8+ZqbxwCNP2jyqXoV/fmhy+WW/2SqFsb1pX68SfEpZ/TCrI3aHzcP//jitodvYmvL+6Xcr5mV
        vb1ScCzRnPRPfz+LsRSWNasuwRrZlh1sx0E8AriddyzEDfE6EkglFhJDJO5u9fJbFJ0etEMB78D5
        4Djm/7kjT0wqhSNURyS+u/2MGJKRu+0ExNkrt1pJti9p2x6b3TBJgmUXuzgnDmI8UWMbkVxeinCw
        Mo311/l/v3rF7+01D+OkZYE0PrbsYAu+sSyxU0jLLtIiYzmBrFiwnCT9FcsdOOK8ZHbFleSn0znP
        nDCnxbnAnGT9JeYtrP+FOcV8nTlNnsoc3bBAD85adtCNRcsSffjBsoseca/lBE7Q09LiJOm/ttyB
        0+IqcwfncJt5q4krO5k7jV7uY+5m7mPebuLKUea7iHvk48w72OYF5rvZT8C8k/WvMN/Dc19j3s02
        bzPvZZv3me9j/ox5P9t/xdzPzPVJcc7yGnPL/1+GO1lPVTXM+VNWOTRRg0YRHgrUK5yj1kvaEA1E
        xAWiCtl4qJL2ADKkG6Q3XxYjzEcR0E9hCj5KtBd1xCxp6jV5mKP7LJBr1nTRK2h1TvU2w0akCmGl
        5lWbBzJqMJsdyaijQaCm/FK5HqspHetoTtMsn4LO0T2mlqcwmlTVOT/28wGhCVKiNANKLiJRlxqB
        F603axQznIzRhDSq6EWZ4UUs+xud0VHsh1U1kMlmNwu9kTuFaRqpURU0VS3PVmZ0iE7gct0MG/8+
        2fmUvKlfRLYmisd1w8pk1LSu1XUlryM1MNTH9epTftWv+16gIh1oL9abJZyjrfF5a4qccp3oFAcz
        Wxxx4DpvlaKKxuytRDzeth5rW4W8qBFesvEX8RFRmLBHoB+TpCmRVCCb1gFCruzHqhhW6+qUF6tC
        pL26nlWN2K+W1LhRjxlVGKmRTFYVo7CiJug09E+GJb+QocMCPMWBK1wvEOfRFF2U0klK8CppqqvG
        pylRc2Zn+XDQWZIL8iO5KC9S+1RekOex1uOyZGR/w/Hf1lhzqVfFsxE39B/ws7Rm3N3nDrhPuMfc
        w3R/aE28KsfY2J+RPNp+j+KaOoCey4h+Dd48b9O5G0v2K7j0AM6s+5WQ/E0wVoK+pA6/3bup7bJf
        CMGjwvxTsr74/f/F95m3TH9x8o0/TU//N+7/D/ScVcA=
        """.encode('latin1')
        uncompressed = zlib.decompress(base64.decodebytes(font))
        ttf = BytesIO(uncompressed)
        setattr(ttf ,"name", "(invisible.ttf)")
        font = TTFont('invisible', ttf)
        for coord in font.face.bbox:
            self.assertEqual(coord, 0)
        pdfmetrics.registerFont(font)
        _simple_subset_generation('test_pdfbase_ttffonts_invisible.pdf',2,fonts=('invisible',))

    def testSplitString(self):
        "Tests TTFont.splitString"
        doc = PDFDocument()
        font = TTFont("Vera", "Vera.ttf", asciiReadable=True)
        T = list(range(256))
        for c in sorted(font.face.charToGlyph):
            if c not in T: T.append(c)
        text = "".join(map(chr,T))
        #we ignore rserveTTFNotDef by assuming it's always True
        #PDFUA forbids index 0(.notdef) in strings
        chunks = [(0,
            b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 !"#$%&\''
            b'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmno'
            b'pqrstuvwxyz{|}~\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x00\x00\x00\x00 \x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f'
            b'\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
            b'\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f'
            b'\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f'
            b'\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf'
            b'\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf'
            b'\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf'
            b'\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf'
            b'\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef'
            b'\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff'),
         (1,
            b'\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f\x10\x11\x12\x13\x14'
            b'\x15\x16\x17\x18\x19\x1a\x1b\x1c')]
        self.assertEqual(font.splitString(text, doc), chunks)
        # Do it twice
        self.assertEqual(font.splitString(text, doc), chunks)

    def testSplitStringSpaces(self):
        # In order for justification (word spacing) to work, the space
        # glyph must have a code 32, and no other character should have
        # that code in any subset, or word spacing will be applied to it.

        doc = PDFDocument()
        font = TTFont("Vera", "Vera.ttf")
        text = b"".join(utf8(i) for i in range(512, -1, -1))
        chunks = font.splitString(text, doc)
        state = font.state[doc]
        self.assertEqual(state.assignments[32], 32)
        self.assertEqual(state.subsets[0][32], 32)

    def testSubsetInternalName(self):
        "Tests TTFont.getSubsetInternalName"
        doc = PDFDocument()
        font = TTFont("Vera", "Vera.ttf")
        # Actually generate some subsets
        text = b"".join(utf8(i) for i in range(513))
        font.splitString(text, doc)
        self.assertRaises(IndexError, font.getSubsetInternalName, -1, doc)
        self.assertRaises(IndexError, font.getSubsetInternalName, 3, doc)
        self.assertEqual(font.getSubsetInternalName(0, doc), "/F1+0")
        self.assertEqual(doc.delayedFonts, [font])

    def testAddObjectsEmpty(self):
        "TTFont.addObjects should not fail when no characters were used"
        font = TTFont("Vera", "Vera.ttf")
        doc = PDFDocument()
        font.addObjects(doc)

    def no_longer_testAddObjectsResets(self):
        "Test that TTFont.addObjects resets the font"
        # Actually generate some subsets
        doc = PDFDocument()
        font = TTFont("Vera", "Vera.ttf")
        font.splitString('a', doc)            # create some subset
        doc = PDFDocument()
        font.addObjects(doc)
        self.assertEqual(font.frozen, 0)
        self.assertEqual(font.nextCode, 0)
        self.assertEqual(font.subsets, [])
        self.assertEqual(font.assignments, {})
        font.splitString('ba', doc)           # should work

    def testParallelConstruction(self):
        "Test that TTFont can be used for different documents at the same time"
        doc1 = PDFDocument()
        doc2 = PDFDocument()
        font = TTFont("Vera", "Vera.ttf", asciiReadable=1)
        self.assertEqual(font.splitString('hello ', doc1), [(0, b'hello ')])
        self.assertEqual(font.splitString('hello ', doc2), [(0, b'hello ')])
        self.assertEqual(font.splitString('\xae\xab', doc1), [(0, b'\x01\x02')])
        self.assertEqual(font.splitString('\xab\xae', doc2), [(0, b'\x01\x02')])
        self.assertEqual(font.splitString('\xab\xae', doc1), [(0, b'\x02\x01')])
        self.assertEqual(font.splitString('\xae\xab', doc2), [(0, b'\x02\x01')])
        font.addObjects(doc1)
        #after addObjects doc1 state is no longer valid, doc2 should be OK
        self.assertEqual(font.splitString('\xae\xab', doc2), [(0, b'\x02\x01')])
        font.addObjects(doc2)

    def testAddObjects(self):
        "Test TTFont.addObjects"
        # Actually generate some subsets
        doc = PDFDocument()
        font = TTFont("Vera", "Vera.ttf", asciiReadable=1)
        font.splitString('a', doc)            # create some subset
        internalName = font.getSubsetInternalName(0, doc)[1:]
        font.addObjects(doc)
        pdfFont = doc.idToObject[internalName]
        self.assertEqual(doc.idToObject['BasicFonts'].dict[internalName], pdfFont)
        self.assertEqual(pdfFont.Name, internalName)
        self.assertEqual(pdfFont.BaseFont, "AAAAAA+BitstreamVeraSans-Roman")
        self.assertEqual(pdfFont.FirstChar, 0)
        self.assertEqual(pdfFont.LastChar, 127)
        self.assertEqual(len(pdfFont.Widths.sequence), 128)
        toUnicode = doc.idToObject[pdfFont.ToUnicode.name]
        self.assertTrue(toUnicode.content != "")
        fontDescriptor = doc.idToObject[pdfFont.FontDescriptor.name]
        self.assertEqual(fontDescriptor.dict['Type'], '/FontDescriptor')

    def testMakeToUnicodeCMap(self):
        "Test makeToUnicodeCMap"
        self.assertEqual(makeToUnicodeCMap("TestFont", [ 0x1234, 0x4321, 0x4242 ]),
"""/CIDInit /ProcSet findresource begin
12 dict begin
begincmap
/CIDSystemInfo
<< /Registry (TestFont)
/Ordering (TestFont)
/Supplement 0
>> def
/CMapName /TestFont def
/CMapType 2 def
1 begincodespacerange
<00> <02>
endcodespacerange
3 beginbfchar
<00> <1234>
<01> <4321>
<02> <4242>
endbfchar
endcmap
CMapName currentdict /CMap defineresource pop
end
end""")

    @rlSkipUnless(uharfbuzz,'no harfbuzz support')
    def test_ShapedStrOps(self):
        sd = ShapedStr('0123456789',shapeData=list(range(10)))
        e = []
        def sdcmp(op,v,ex):
            e.append(f'{op} errors:')
            i=len(e)
            if v!=ex: e.append(f'{v!a}!={ex!a}')
            if type(v)!=type(ex):
                e.append(f'{v.__class__.__name__}!={ex.__class__.__name__}')
            elif isinstance(ex,ShapedStr) and v.__shapeData__!=ex.__shapeData__:
                e.append(f'{v.__shapeData__}!={ex.__shapeData__}')
            if len(e)==i:
                e.pop()
            else:
                e[i-1] = ' '.join(e[i-1:])
                del e[i:]
        sdcmp('sd[1]',sd[1],ShapedStr('1',[1]))
        sdcmp('sd[2]',sd[2],ShapedStr('2',[2]))
        sdcmp('sd[-1]',sd[-1],ShapedStr('9',[9]))
        sdcmp('sd[:2]',sd[:2],ShapedStr('01',[0,1]))
        sdcmp('sd[:-8]',sd[:-8],ShapedStr('01',[0,1]))
        sdcmp('sd[-3:-2]',sd[-3:-2],ShapedStr('7',[7]))
        sdcmp('sd[:0]',sd[:0],'')
        sdcmp('sd[1:1]',sd[1:1],'')
        sdcmp('sd[-2:-2]',sd[-2:-2],'')
        sdcmp('sd[:2]+sd[-2:]',sd[:2]+sd[-2:],ShapedStr('0189',[0,1,8,9]))
        sdcmp('sd+"ABC"',sd+"ABC",ShapedStr('0123456789ABC',list(range(10))+3*[_sdSimple]))
        sdcmp('"ABC"+sd',"ABC"+sd,ShapedStr('ABC0123456789',3*[_sdSimple]+list(range(10))))
        sd += 'ABC'
        sdcmp('sd+="ABC"',sd,ShapedStr('0123456789ABC',list(range(10))+3*[_sdSimple]))
        e = '\n'.join(e)
        self.assertEqual('',e)

    def hbIfaceTest(self, ttfpath, text, exLen, exText, exShapedData):
        ttfn = os.path.splitext(os.path.basename(ttfpath))[0]
        ttf = freshTTFont(ttfn, ttfpath)
        fontName = ttf.fontName
        fontSize = 30
        pdfmetrics.registerFont(ttf)
        w = [pdfmetrics.stringWidth(text,fontName,fontSize),(ABag(fontName=fontName,fontSize=fontSize),text)]
        new = shapeFragWord(w)
        ttf.unregister()
        self.assertEqual(len(new),2,'expected a len of 2')
        self.assertTrue(isinstance(new,ShapedFragWord),f'returned list class is {new.__class__.__name__} not expected ShapedFragWord')
        self.assertEqual(new[0],exLen,f'new[0]={new[0]} not expected ={exLen}')
        self.assertTrue(isinstance(new[1][1],ShapedStr),f'returned str class is {new[1].__class__.__name__} not expected ShapedStr')
        self.assertTrue(new[1][1]==exText,'shaped string is wrong')
        self.assertEqual(new[1][1].__shapeData__,exShapedData, 'shape data is wrong')

    @rlSkipUnless(uharfbuzz,'no harfbuzz support')
    def test_hb_shape_change(self):
        return self.hbIfaceTest('hb-test.ttf','\u1786\u17D2\u1793\u17B6\u17C6|', 44.22,'\ue000\ue001\u17c6|',
                                [ShapeData(cluster=0, x_advance=923, y_advance=0, x_offset=0, y_offset=0, width=923),
                                ShapeData(cluster=0, x_advance=0, y_advance=0, x_offset=-296, y_offset=-26, width=0),
                                ShapeData(cluster=4, x_advance=0, y_advance=0, x_offset=47, y_offset=-29, width=0),
                                ShapeData(cluster=5, x_advance=551, y_advance=0, x_offset=0, y_offset=0, width=551)])

    @rlSkipUnless(uharfbuzz,'no harfbuzz support')
    def test_hb_ligature(self):
        #ligatures cause the standard length 133.2275390625 to be reduced to 130.78125
        return self.hbIfaceTest('Vera.ttf','Aon Way',130.78125,'Aon Way',
                                [ShapeData(cluster=0, x_advance=675.29296875, y_advance=0, x_offset=0.0, y_offset=0, width=684.08203125),
                                ShapeData(cluster=1, x_advance=603.02734375, y_advance=0, x_offset=-8.7890625, y_offset=0, width=611.81640625),
                                ShapeData(cluster=2, x_advance=633.7890625, y_advance=0, x_offset=0.0, y_offset=0, width=633.7890625),
                                ShapeData(cluster=3, x_advance=317.87109375, y_advance=0, x_offset=0.0, y_offset=0, width=317.87109375),
                                ShapeData(cluster=4, x_advance=956.54296875, y_advance=0, x_offset=0.0, y_offset=0, width=988.76953125),
                                ShapeData(cluster=5, x_advance=581.0546875, y_advance=0, x_offset=-31.73828125, y_offset=0, width=612.79296875),
                                ShapeData(cluster=6, x_advance=591.796875, y_advance=0, x_offset=0.0, y_offset=0, width=591.796875)])


    @staticmethod
    def drawVLines(canv,x,y,fontSize,X):
        canv.saveState()
        canv.setLineWidth(0.5)
        canv.setStrokeColor((1,0,0))
        canv.lines([(_+x,y-0.2*fontSize,_+x,y+fontSize) for _ in X])
        canv.restoreState()

    @classmethod
    def drawParaVLines(cls,canv,x,y,p,ttfn):
        X = [0]
        mfs = 0
        frags = p.frags
        for f in frags:
            for t in f[1:]:
                fontName = t[0].fontName
                if fontName!=ttfn: continue
                fontSize = t[0].fontSize
                mfs = max(mfs,fontSize)
                v = t[1]
                if type(v) is str:
                    for s in v:
                        X.append(X[-1]+pdfmetrics.stringWidth(s,fontName,fontSize))
                else:
                    for d in v.__shapeData__:
                        X.append(X[-1]+d.x_advance*fontSize/1000)
            if fontName!=ttfn: break
            if isinstance(f,_HSFrag):
                X.append(X[-1]+pdfmetrics.stringWidth(' ',fontName,fontSize))
        if mfs: cls.drawVLines(canv,x,y,mfs,X)

    @staticmethod
    def shapedStrAdvances(s,fontSize):
        A = [0]
        for a in s.__shapeData__:
            A.append(fontSize*a.x_advance/1000 + A[-1])
        return A

    @rlSkipUnless(uharfbuzz,'no harfbuzz support')
    def test_hb_examples(self):
        canv = Canvas(outputfile('test_pdfbase_ttfonts_hb_examples.pdf'))
        def hb_example(ttfpath, text, y, fpdfLiteral, excode):
            ttfn = os.path.splitext(os.path.basename(ttfpath))[0]
            ttf = freshTTFont(ttfn, ttfpath, shapable=False)
            try:
                fontName = ttf.fontName
                fontSize = 30
                pdfmetrics.registerFont(ttf)
                ttf.splitString(text,canv._doc)
                w = [pdfmetrics.stringWidth(text,fontName,fontSize),(ABag(fontName=fontName,fontSize=fontSize),text)]
                new = shapeFragWord(w)
                advances = self.shapedStrAdvances(new[1][1],fontSize)
                canv.setFont('Helvetica',12)
                x1 = 400
                canv.drawString(x1,y,f'unshaped {fontName}')
                canv.drawString(x1,y-1.2*fontSize,f'fpdf2 shaping {fontName}')
                canv.drawString(x1,y-2*1.2*fontSize,f'rl shaping {fontName}')
                canv.setFont(fontName,fontSize)
                canv.drawString(36,y,text)
                self.drawVLines(canv,36,y,fontSize,
                   [pdfmetrics.stringWidth(text[:_],fontName,fontSize) for _ in range(len(text)+1)])
                canv.saveState()
                canv.translate(0,-fontSize*1.2)
                ttf.shapable = True
                t = canv.beginText(36, y-1.2*fontSize) #786 - 1.2*30
                t._textOut(new[1][1],False)
                code = t.getCode()
                canv.drawText(t)
                self.drawVLines(canv,36,y-1.2*fontSize,fontSize,advances)
                if fpdfLiteral:
                    canv.addLiteral(fpdfLiteral)
                    self.drawVLines(canv,36,y,fontSize,advances)
                canv.restoreState()
                if code!=excode:
                    canv.showPage()
                    canv.save()
                self.assertEqual(code,excode,f'{fontName} PDF _textOut is wrong\n')
            finally:
                ttf.unregister()
        hb_example(
            'Vera.ttf',
            'Aon Way',
            786,
            fpdfLiteral='''BT 1 0 0 1 36 786 Tm /F2+0 30 Tf 36 TL (A) Tj 1 0 0 1 56 786 Tm (o) Tj 1 0 0 1 74.35 786 Tm (n) Tj 1 0 0 1 93.36 786 Tm ( ) Tj 1 0 0 1 102.9 786 Tm (W) Tj 1 0 0 1 130.64 786 Tm (a) Tj 1 0 0 1 149.03 786 Tm (y) Tj 1 0 0 1 166.78 786 Tm ET''',
            excode='''BT 1 0 0 1 36 750 Tm /F2+0 30 Tf 36 TL [(A) 17.57812 (on W) 63.96484 (ay)] TJ ET'''
            )
        canv.translate(0,-36)
        hb_example(
            'hb-test.ttf',
            '\u1786\u17D2\u1793\u17B6\u17C6|',
            706,
            fpdfLiteral=r'''BT 1 0 0 1 0 0 Tm 1 0 0 1 36 706 Tm /F3+0 30 Tf 36 TL (\006) Tj 1 0 0 1 54.81 705.22 Tm (\007) Tj 1 0 0 1 63.69 706 Tm 1 0 0 1 65.1 705.13 Tm (\005) Tj 1 0 0 1 63.49 706 Tm (|) Tj  ET''',
            excode = r'''BT 1 0 0 1 36 670 Tm /F3+0 30 Tf 36 TL (\006) Tj -0.78 Ts [296 (\007) -296] TJ -0.87 Ts [-47 (\005) 47 (|)] TJ ET'''
            )
        canv.translate(0,-50)
        if haveDejaVuSans:
            hb_example(
                'DejaVuSans.ttf',
                'Huffing Clifftop finish|',
                706-70,
                fpdfLiteral=r'''BT 1 0 0 1 36 636 Tm /F4+0 30 Tf 36 TL (H) Tj 1 0 0 1 58.56 636 Tm (u) Tj 1 0 0 1 77.57 636 Tm (\001) Tj 1 0 0 1 106.58 636 Tm (n) Tj 1 0 0 1 125.59 636 Tm (g) Tj 1 0 0 1 144.63 636 Tm ( ) Tj 1 0 0 1 154.17 636 Tm (C) Tj 1 0 0 1 175.12 636 Tm (l) Tj 1 0 0 1 183.45 636 Tm (i) Tj 1 0 0 1 191.79 636 Tm (\002) Tj 1 0 0 1 212.46 636 Tm (t) Tj 1 0 0 1 224.22 636 Tm (o) Tj 1 0 0 1 242.57 636 Tm (p) Tj 1 0 0 1 261.62 636 Tm ( ) Tj 1 0 0 1 271.15 636 Tm (\003) Tj 1 0 0 1 290.05 636 Tm (n) Tj 1 0 0 1 309.06 636 Tm (i) Tj 1 0 0 1 317.4 636 Tm (s) Tj 1 0 0 1 333.03 636 Tm (h) Tj 1 0 0 1 352.04 636 Tm (|) Tj 1 0 0 1 362.15 636 Tm ET''',
                excode=r'''BT 1 0 0 1 36 600 Tm /F4+0 30 Tf 36 TL (Hu\001ng Cli\002top \003nish|) Tj ET'''
                )

        canv.showPage()
        canv.save()

    def hb_paragraph_drawon(self, canv, ttfpath, text, y=786, textp=None):
        try:
            ttfn = os.path.splitext(os.path.basename(ttfpath))[0]
            ttfn1 = ttfn+'1'
            ttf = freshTTFont(ttfn,ttfpath,shapable=False)
            ttf1 = freshTTFont(ttfn1,ttfpath,shapable=True)
            ttf1.face.name = ttf.face.name + b'1'
            pdfmetrics.registerFont(ttf)
            pdfmetrics.registerFont(ttf1)
            stysh = getSampleStyleSheet()
            fontSize = 30
            leading = 36
            sty = stysh.Normal.clone('sty',fontName=ttfn,fontSize=fontSize,leading=leading)
            sty1 = stysh.Normal.clone('sty1',fontName=ttfn1,fontSize=fontSize,leading=leading,shaping=True)
            pW, pH = canv._pagesize
            x = 36
            aW = pW - 2*36
            aH = pH - 2*36
            p = Paragraph(f'{text}<span face="Helvetica" size="12"> {ttfn} unshaped</span>',sty)
            w,h = p.wrap(aW,aH)
            p.drawOn(canv, x=x, y=y)
            self.drawParaVLines(canv,x,y,p,ttfn)
            y -= leading
            p1 = Paragraph(f'{text}<span face="Helvetica" size="12"> {ttfn} shaped</span>',sty1)
            w,h = p1.wrap(aW,aH)
            p1.drawOn(canv, x=x, y=y)
            self.drawParaVLines(canv,x,y,p1,ttfn1)
            y -= 12
            xe = pW - 36
            xm = pW*0.5
            canv.saveState()
            if textp is None: textp = text
            canv.setFont(ttfn,10)
            canv.drawString(x,y,textp)
            canv.drawRightString(xe,y,textp)
            canv.drawCentredString(xm,y,textp)
            y -= 12
            canv.setFont(ttfn1,10)
            canv.drawString(x,y,textp,shaping=True)
            canv.drawRightString(xe,y,textp,shaping=True)
            canv.drawCentredString(xm,y,textp,shaping=True)
            canv.restoreState()
            y -= 12
        finally:
            for _ in ('ttf','ttf1'):
                if _ in locals():
                    locals()[_].unregister()

    @rlSkipUnless(uharfbuzz,'no harfbuzz support')
    def test_hb_paragraph_drawOn(self):
        canv = Canvas(outputfile('test_pdfbase_ttfonts_hb_para_drawOn.pdf'))
        self.hb_paragraph_drawon(canv, 'hb-test.ttf', '\u1786\u17D2\u1793\u17B6\u17C6|', y=786)
        self.hb_paragraph_drawon(canv, 'Vera.ttf', 'Aon Way|', y=786-2*48)
        if haveDejaVuSans:
            self.hb_paragraph_drawon(canv, 'DejaVuSans.ttf',
                            '<span color="green">H</span>u<span color="blue">f</span>fing <u>Clifftop</u> <a href="https://www.reportlab.com">finish</a>|',
                            y=786-4*48, textp='Huffing Clifftop finish|')
        canv.showPage()
        canv.save()

def makeSuite():
    suite = makeSuiteForClasses(
        TTFontsTestCase,
        TTFontFileTestCase,
        TTFontFaceTestCase,
        TTFontTestCase)
    return suite


#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
"""Tests for reportlab.lib.rl_safe_eval
"""
__version__='3.5.33'
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, printLocation
setOutDir(__name__)
import os, time, sys
import reportlab
from reportlab import rl_config
import unittest
from reportlab.lib import colors
from reportlab.lib.utils import rl_safe_eval, rl_safe_exec, annotateException, rl_extended_literal_eval
from reportlab.lib.rl_safe_eval import BadCode

testObj = [1,('a','b',2),{'A':1,'B':2.0},"32"]
class TestClass:
    a = 1
    format = 3
testInst = TestClass()
def testFunc(bad=False):
    return open('/tmp/myfile','r') if bad else testObj

class SafeEvalTestSequenceMeta(type):
    def __new__(cls, name, bases, cdict):
        def genTest(kind, expr,**kwds):
            def test(self):
                getattr(self,kind+'s')(expr,**kwds)
            return test

        for kind, _data in (
                (
                'work',
                (
                '[i for i in range(10)]',
                '3**4',
                '3*"A"',
                '3+4',
                '(3,4)',
                'SafeEvalTestCase',
                ("testObj",dict(g=dict(testObj=testObj))),
                "(lambda x,y:[x,y])('a',2)",
                "(lambda x:[x])(2)",
                "(lambda *args:[args])(2)",
                "(lambda y: (lambda f:f(y))(lambda x: x+1))(2)",
                "(lambda f: lambda n: (1,(1,(1,(1,f(n-1))))) if n else 1)(lambda x:x)(5)",
                "((lambda f: (lambda x: x(x))(lambda y: f(lambda *args: y(y)(*args)))) (lambda f: lambda n: (1,(1,(1,(1,f(n-1))))) if n else 1)(30))",
                'list(range(1000))[1:1000:100]',
                'tuple(range(1000))[1:1000:100]',
                'dict(a=1)["a"]',
                'dict(a=1).setdefault("a",2)',
                'dict(a=1).get("a",2)',
                'dict(a=1).pop("a",2)',
                '{"_":1+_ for _ in (1,2)}.pop(1,None)',
                '1 if True else "a"',
                '1 if False else "a"',
                'testFunc(bad=False)',
                '(min([1]),min(1,2),max([1]),max(1,2))',
                '(sum((1,2,3)),sum((1,2,3),-1))',
                'list(enumerate((1,2,3)))',
                'list(zip((1,2,3),("a","b","c")))',
                '(hasattr(testInst,"b"),hasattr(testInst,"a"))',
                'list(map(lambda x: (x+13,chr(x)),(1,2,3,4)))',
                '(any([1]),any([]),all([]),all([1,None]),all([1,2]))',
                '(getattr(testInst,"a"),getattr(testInst,"a",12),getattr(testInst,"xxx",13))',
                'list(sorted([3,4,1,2,0],reverse=True))',
                'list(reversed([3,4,1,2,0]))',
                'list(range(1,10,3))',
                '({1,2,3},set([4,5,6]),frozenset([7,8,9]),{i for i in range(1,10,3)})',
                '"%s%s" % (1,2)',
                )
                ),
                (
                'fail',
                (
                'vars()',
                '(type(1),type(str),type(testObj),type(TestClass))',
                'open("/tmp/myfile")',
                'SafeEvalTestCase.__module__',
                ("testInst.__class__.__bases__[0].__subclasses__()",dict(g=dict(testInst=testInst))),
                "10**200**200",
                "pow(10,200)",
                '__import__("reportlab")',
                ('(lambda i: [i for i in ((i, 1) for j in range(1000000))][-1])(1)',dict(timeout=0.1)),
                ('[i for i in ((j, 1) for j in range(1000000))][-1]',dict(timeout=0.1)),
                ('''((lambda f: (lambda x: x(x))(lambda y: f(lambda *args: y(y)(*args)))) (lambda f: lambda n: (1,(1,(1,(1,f(n-1))))) if n else 1)(300))''',
                        dict(timeout=0.1,exception=RuntimeError)),
                'dict(__class__=1)["__class__"]',
                'dict(a=1).setdefault("__class__",2)',
                'dict(a=1).get("__class__",2)',
                'dict(a=1).pop("__class__",2)',
                '{"__class__":1}["__class__"]',
                '{"__class__":1}.setdefault("__class__",2)',
                '{"__class__":1}.get("__class__",2)',
                '{"__class__":1}.pop("__class__",2)',
                '{"_":1 for _ in (1,2)}.pop("__class__",2)',
                'type("Devil",[dict],{__init__:lambda self:self.__class__})',
                'testFunc(bad=True)',
                'getattr(testInst,"__class__",14)',
                '"{1}{2}".format(1,2)',
                'builtins',
                '[ [ [ [ ftype(ctype(0, 0, 0, 0, 3, 67, b"t\\x00d\\x01\\x83\\x01\\xa0\\x01d\\x02\\xa1\\x01\\x01\\x00d\\x00S\\x00", (None, "os", "touch /tmp/exploited"), ("__import__", "system"), (), "<stdin>", "", 1, b"\\x12\\x01"), {})() for ftype in [type(lambda: None)] ] for ctype in [type(getattr(lambda: {None}, Word("__code__")))] ] for Word in [orgTypeFun("Word", (str,), { "mutated": 1, "startswith": lambda self, x: False, "__eq__": lambda self,x: self.mutate() and self.mutated < 0 and str(self) == x, "mutate": lambda self: {setattr(self, "mutated", self.mutated - 1)}, "__hash__": lambda self: hash(str(self)) })] ] for orgTypeFun in [type(type(1))]] and "red"',
                )
                ),
                ):
            tfmt = 'test_ExpectedTo%s_%%02d' % kind.capitalize()
            for i, expr in enumerate(_data):
                if expr is None:
                    continue #test = genTest('skip','')
                else:
                    expr, kwds = expr if isinstance(expr,tuple) else (expr,{})
                    test = genTest(kind, expr,**kwds)
                cdict[tfmt%i] = test
        return type.__new__(cls, name, bases, cdict)

def addMeta(mcs):
    def wrap(cls):
        ov = cls.__dict__.copy()
        sl = ov.get('__slots__')
        if sl is not None:
            if isinstance(sl, str):
                sl = [sl]
            for slv in sl:
                ov.pop(slv)
        ov.pop('__dict__', None)
        ov.pop('__weakref__', None)
        if hasattr(cls, '__qualname__'):
            ov['__qualname__'] = cls.__qualname__
        return mcs(cls.__name__, cls.__bases__, ov)
    return wrap

@addMeta(SafeEvalTestSequenceMeta)
class SafeEvalTestCase(unittest.TestCase):
    def works(self, expr, g=None, l=None):
        try:
            answer = eval(expr,g,l)
            result = rl_safe_eval(expr,g,l)
        except:
            print('expr=%r' % expr)
            annotateException('\nexpr=%r\n' % expr)
        self.assertEqual(answer,result,f"rl_safe_eval({expr!r}) = {result!r}) not expected {answer!r}")
    def skips(self,*args,**kwds):
        raise unittest.SkipTest
    def fails(self, expr, g=None, l=None, timeout=None, exception=BadCode):
        try:
            result = rl_safe_eval(expr,g,l,timeout=timeout)
            self.assertEqual(True,False,f"rl_safe_eval({expr!r}) = {result!r} did not raise {exception.__name__}")
        except exception:
            return
        except:
            self.assertEqual(True,False,"rl_safe_eval({expr!r}) raised {sys.exc_info()[0].__name__}: {str(sys.exc_info()[1])} instead of {exception.__name__}")

GA = 'ga'
class SafeEvalTestBasics(unittest.TestCase):
    def test_001(self):
        A=3
        self.assertTrue(rl_safe_eval("A==3"))
    def test_002(self):
        self.assertTrue(rl_safe_eval("GA=='ga'"))

class SafeExecTestBasics(unittest.TestCase):
    def test_funcdef_blocks(self):
        '''test security issue found by Ethan Kim lt ethan 4t cremit d0t io gt'''
        codes = (
            '().__class__.__bases__[0]',
            'def f():\n'
             '\tfor c in ().__class__.__bases__[0].__subclasses__():\n'
              '\t\tif c.__name__ == "_wrap_close":\n'
               '\t\t\treturn c.__init__.__globals__["system"]("id")\n'
            'f()\n',
            )
        blocked = 0
        for c in codes:
            try:
                rl_safe_exec(c,{},{})
            except BadCode:
                blocked += 1
        self.assertEqual(blocked,2,f'{blocked=} expected 2; probably BadCode detection in FunctionDef fails')

class ExtendedLiteralEval(unittest.TestCase):
    def test_001(self):
        S = colors.getAllNamedColors().copy()
        C = {s:getattr(colors,s) for s in '''Blacker CMYKColor CMYKColorSep Color ColorType HexColor PCMYKColor PCMYKColorSep Whiter
                        _chooseEnforceColorSpace _enforceCMYK _enforceError _enforceRGB _enforceSEP _enforceSEP_BLACK
                        _enforceSEP_CMYK _namedColors _re_css asNative cmyk2rgb cmykDistance color2bw colorDistance
                        cssParse describe fade fp_str getAllNamedColors hsl2rgb hue2rgb linearlyInterpolatedColor
                        obj_R_G_B opaqueColor rgb2cmyk setColors toColor toColorOrNone'''.split()
                        if callable(getattr(colors,s,None))}
        def showVal(s):
            try:
                r = rl_extended_literal_eval(s,C,S)
            except:
                r = str(sys.exc_info()[1])
            return r

        for expr, expected in (
                ('1.0', 1.0),
                ('1', 1),
                ('red', colors.red),
                ('True', True),
                ('False', False),
                ('None', None),
                ('Blacker(red,0.5)', colors.Color(.5,0,0,1)),
                ('PCMYKColor(21,10,30,5,spotName="ABCD")', colors.PCMYKColor(21,10,30,5,spotName='ABCD',alpha=100)),
                ('HexColor("#ffffff")', colors.Color(1,1,1,1)),
                ('linearlyInterpolatedColor(red, blue, 0, 1, 0.5)', colors.Color(.5,0,.5,1)),
                ('red.rgb()', 'Bad expression'),
                ('__import__("sys")', 'Bad expression'),
                ('globals()', 'Bad expression'),
                ('locals()', 'Bad expression'),
                ('vars()', 'Bad expression'),
                ('builtins', 'Bad expression'),
                ('__file__', 'Bad expression'),
                ('__name__', 'Bad expression'),
                ):
            self.assertEqual(showVal(expr),expected,f"rl_extended_literal_eval({expr!r}) is not equal to expected {expected}")

def makeSuite():
    return makeSuiteForClasses(SafeEvalTestCase,SafeEvalTestBasics,SafeExecTestBasics,ExtendedLiteralEval)

if __name__ == "__main__": #noruntests
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, Line, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=200,height=100,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(24.5,10,18,42.5,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(69.5,10,18,56.66667,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(114.5,10,18,70.83333,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(159.5,10,18,85,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(87.5,10,18,14.16667,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(132.5,10,18,21.25,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(177.5,10,18,28.33333,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,10,200,10,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,10,20,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(65,10,65,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(110,10,110,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(155,10,155,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(200,10,200,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,10,20,95,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,10,15,10,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,24.16667,15,24.16667,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,38.33333,15,38.33333,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,52.5,15,52.5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,66.66667,15,66.66667,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,80.83333,15,80.83333,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,95,15,95,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,10)
		v0.add(String(-10,-4,'70',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,24.16667)
		v0.add(String(-10,-4,'80',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,38.33333)
		v0.add(String(-10,-4,'90',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,52.5)
		v0.add(String(-15,-4,'100',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,66.66667)
		v0.add(String(-15,-4,'110',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,80.83333)
		v0.add(String(-15,-4,'120',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,95)
		v0.add(String(-15,-4,'130',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)
